import uuid
from datetime import datetime
from typing import Optional
from cpl_core.database import TableABC

from bot_data.model.auth_role_enum import AuthRoleEnum
from bot_data.model.server import Server


class AuthUser(TableABC):

    def __init__(
            self,
            first_name: str,
            last_name: str,
            email: str,
            password: str,
            password_salt: Optional[str],
            refresh_token: Optional[str],
            confirmation_id: Optional[str],
            forgot_password_id: Optional[str],
            oauth_id: Optional[str],
            refresh_token_expire_time: datetime,
            auth_role: AuthRoleEnum,
            user_id: Optional[int],
            created_at: datetime = None,
            modified_at: datetime = None,
            id=0
    ):
        self._auth_user_id = id
        self._first_name = first_name
        self._last_name = last_name
        self._email = email
        self._password = password
        self._password_salt = uuid.uuid4() if password_salt is None else password_salt
        self._refresh_token = refresh_token
        self._confirmation_id = confirmation_id
        self._oauth_id = oauth_id
        self._forgot_password_id = forgot_password_id
        self._refresh_token_expire_time = refresh_token_expire_time

        self._auth_role_id = auth_role
        self._user_id = user_id

        TableABC.__init__(self)
        self._created_at = created_at if created_at is not None else self._created_at
        self._modified_at = modified_at if modified_at is not None else self._modified_at

    @property
    def id(self) -> int:
        return self._auth_user_id

    @property
    def first_name(self) -> str:
        return self._first_name

    @first_name.setter
    def first_name(self, value: str):
        self._first_name = value

    @property
    def last_name(self) -> str:
        return self._last_name

    @last_name.setter
    def last_name(self, value: str):
        self._last_name = value

    @property
    def email(self) -> str:
        return self._email

    @email.setter
    def email(self, value: str):
        self._email = value

    @property
    def password(self) -> str:
        return self._password

    @password.setter
    def password(self, value: str):
        self._password = value

    @property
    def password_salt(self) -> str:
        return self._password_salt

    @password_salt.setter
    def password_salt(self, value: str):
        self._password_salt = value

    @property
    def refresh_token(self) -> Optional[str]:
        return self._refresh_token

    @refresh_token.setter
    def refresh_token(self, value: Optional[str]):
        self._refresh_token = value

    @property
    def confirmation_id(self) -> Optional[str]:
        return self._confirmation_id

    @confirmation_id.setter
    def confirmation_id(self, value: Optional[str]):
        self._confirmation_id = value

    @property
    def forgot_password_id(self) -> Optional[str]:
        return self._forgot_password_id

    @forgot_password_id.setter
    def forgot_password_id(self, value: Optional[str]):
        self._forgot_password_id = value

    @property
    def oauth_id(self) -> Optional[str]:
        return self._oauth_id

    @oauth_id.setter
    def oauth_id(self, value: Optional[str]):
        self._oauth_id = value

    @property
    def refresh_token_expire_time(self) -> datetime:
        return self._refresh_token_expire_time

    @refresh_token_expire_time.setter
    def refresh_token_expire_time(self, value: datetime):
        self._refresh_token_expire_time = value

    @property
    def auth_role(self) -> AuthRoleEnum:
        return self._auth_role_id

    @auth_role.setter
    def auth_role(self, value: AuthRoleEnum):
        self._auth_role_id = value

    @property
    def user_id(self) -> Optional[int]:
        return self._user_id

    @user_id.setter
    def user_id(self, value: Optional[int]):
        self._user_id = value

    @staticmethod
    def get_select_all_string() -> str:
        return str(f"""
            SELECT * FROM `AuthUsers`;
        """)

    @staticmethod
    def get_select_by_id_string(id: int) -> str:
        return str(f"""
            SELECT * FROM `AuthUsers`
            WHERE `Id` = {id};
        """)

    @staticmethod
    def get_select_by_email_string(email: str) -> str:
        return str(f"""
            SELECT * FROM `AuthUsers`
            WHERE `EMail` = '{email}';
        """)

    @staticmethod
    def get_select_by_confirmation_id_string(id: str) -> str:
        return str(f"""
            SELECT * FROM `AuthUsers`
            WHERE `ConfirmationId` = '{id}';
        """)

    @staticmethod
    def get_select_by_forgot_password_id_string(id: str) -> str:
        return str(f"""
            SELECT * FROM `AuthUsers`
            WHERE `ForgotPasswordId` = '{id}';
        """)

    @property
    def insert_string(self) -> str:
        return str(f"""
            INSERT INTO `AuthUsers` (
                `Id`,
                `FirstName`,
                `LastName`,
                `EMail`,
                `Password`,
                `PasswordSalt`,
                `RefreshToken`,
                `ConfirmationId`,
                `ForgotPasswordId`,
                `OAuthId`,
                `RefreshTokenExpiryTime`,
                `AuthRole`,
                `UserId`,
                `CreatedOn`,
                `LastModifiedOn`
                ) VALUES (
                    {self._auth_user_id}, 
                    '{self._first_name}',
                    '{self._last_name}',
                    '{self._email}',
                    '{self._password}', 
                    '{self._password_salt}', 
                    '{"NULL" if self._refresh_token is None else self._refresh_token}',
                    '{"NULL" if self._confirmation_id is None else self._confirmation_id}',
                    '{"NULL" if self._forgot_password_id is None else self._forgot_password_id}',
                    '{"NULL" if self._oauth_id is None else self._oauth_id}',
                    '{self._refresh_token_expire_time}',
                    {self._auth_role_id.value},
                    {"NULL" if self._user_id is None else self._user_id},
                    '{self._created_at}', 
                    '{self._modified_at}'
                )
        """)

    @property
    def udpate_string(self) -> str:
        return str(f"""
            UPDATE `AuthUsers`
            SET `FirstName` = '{self._first_name}', 
            `LastName` = '{self._last_name}', 
            `EMail` = '{self._email}', 
            `Password` = '{self._password}', 
            `PasswordSalt` = '{self._password_salt}', 
            `RefreshToken` = '{self._refresh_token}', 
            `ConfirmationId` = '{"NULL" if self._confirmation_id is None else self._confirmation_id}', 
            `ForgotPasswordId` = '{"NULL" if self._forgot_password_id is None else self._forgot_password_id}',
            `OAuthId` = '{"NULL" if self._oauth_id is None else self._oauth_id}',
            `RefreshTokenExpiryTime` = '{self._refresh_token_expire_time}',
            `AuthRole` = {self._auth_role_id.value},
            `UserId` = {"NULL" if self._user_id is None else self._user_id},
            `LastModifiedOn` = '{self._modified_at}'
            WHERE `AuthUsers`.`Id` = {self._auth_user_id};
        """)

    @property
    def delete_string(self) -> str:
        return str(f"""
            DELETE FROM `AuthUsers`
            WHERE `Id` = {self._auth_user_id};
        """)
